//
//  appController.m
//  serialRulez
//
//  Created by uros petrevski on 10/02/09.
//  Copyright 2009 NoDesign. All rights reserved.
//

#import "AppController.h"
#import "AMSerialPortList.h"
#import "AMSerialPortAdditions.h"

@implementation AppController


- (void)awakeFromNib
{
	// register for port add/remove notification
	[[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(didAddPorts:) name:AMSerialPortListDidAddPortsNotification object:nil];
	[[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(didRemovePorts:) name:AMSerialPortListDidRemovePortsNotification object:nil];
	[AMSerialPortList sharedPortList]; // initialize port list to arm notifications
	
	// get an port enumerator
	NSEnumerator *enumerator = [AMSerialPortList portEnumerator];
	AMSerialPort *aPort;
	while (aPort = [enumerator nextObject]) {
		// print port name
		NSLog(@" %@", [aPort bsdPath]);
		[device addItemWithTitle:[aPort bsdPath]];
		
	}
	
	keynote = [[KeynoteCommands alloc] init];
	bufferEnterStr = [[NSMutableString alloc] init];
	
	sendDifferencies = NO;
	[device setNeedsDisplay:YES];
	[self initPort]; // init at start
}


- (AMSerialPort *)port
{
    return port;
}

- (void)setPort:(AMSerialPort *)newPort
{
    id old = nil;
	
    if (newPort != port) {
        old = port;
        port = [newPort retain];
        [old release];
    }
}


- (void)initPort
{
	deviceName = [[device selectedItem] title];
	
	if (![deviceName isEqualToString:[port bsdPath]]) {
		[port close];
		
		[self setPort:[[[AMSerialPort alloc] init:deviceName withName:deviceName type:(NSString*)CFSTR(kIOSerialBSDModemType)] autorelease]];
		
		// register as self as delegate for port
		[port setDelegate:self];
	
		
		[infos setStringValue:@"attempting to open port\r"];
		[infos setNeedsDisplay:YES];
		[infos displayIfNeeded];
		
		// open port - may take a few seconds ...
		if ([port open]) {
			
			[infos setStringValue:[NSString stringWithFormat: @"port opened at %d bps\r", [port speed] ] ];
			
			[playButt setEnabled:YES];
			[playButt setNeedsDisplay:YES];
			
			
			[infos setNeedsDisplay:YES];
			[infos displayIfNeeded];
			
			// listen for data in a separate thread
			[port readDataInBackground];
			
		} else { // an error occured while creating port
			
			[playButt setEnabled:NO];
			[playButt setNeedsDisplay:YES];
			
			[infos setStringValue:@"couldn't open port for device "];
			[infos setStringValue:deviceName];
			[infos setStringValue:@"\r"];
			[infos setNeedsDisplay:YES];
			[infos displayIfNeeded];
			[self setPort:nil];
		}
	}
}


- (IBAction)chooseDevice:(id)sender
{
	// new device selected
	[self initPort];
}



- (void)didAddPorts:(NSNotification *)theNotification
{
    [infos setStringValue:@"something added check in the list"];
	[infos setNeedsDisplay:YES];
	
	// get an port enumerator
	NSEnumerator *enumerator = [AMSerialPortList portEnumerator];
	AMSerialPort *aPort;
	while (aPort = [enumerator nextObject]) {
		// print port name
		NSLog(@" %@", [aPort bsdPath]);
		[device addItemWithTitle:[aPort bsdPath]];
	}
	[device setNeedsDisplay:YES];
	
}

- (void)didRemovePorts:(NSNotification *)theNotification
{
	
	[infos setStringValue:@"something removed"];
	
	// get an port enumerator
	NSEnumerator *enumerator = [AMSerialPortList portEnumerator];
	AMSerialPort *aPort;
	while (aPort = [enumerator nextObject]) {
		// print port name
		NSLog(@" %@", [aPort bsdPath]);
		[device addItemWithTitle:[aPort bsdPath]];
		
	}
	
	[device setNeedsDisplay:YES];
	
}



- (IBAction)listDevices:(id)sender
{
	// get an port enumerator
	NSEnumerator *enumerator = [AMSerialPortList portEnumerator];
	AMSerialPort *aPort;
	while (aPort = [enumerator nextObject]) {
		// print port name
		NSLog(@" %@", [aPort bsdPath]);
		[device addItemWithTitle:[aPort bsdPath]];
	}
	[device setNeedsDisplay:YES];
}


- (void)serialPortReadData:(NSDictionary *)dataDictionary
{
	// this method is called if data arrives 
	// @"data" is the actual data, @"serialPort" is the sending port
	AMSerialPort *sendPort = [dataDictionary objectForKey:@"serialPort"];
	NSData *data = [dataDictionary objectForKey:@"data"];
	
	
	if ([data length] > 0) {
		NSString *myDataString = [[NSString alloc] initWithData:data encoding:NSASCIIStringEncoding];
				
		BOOL isOKA = NO; // message is ready to be sended ??
		
		NSString *val = [[NSString alloc] init];
		
		for (int i=0; i<[myDataString length]; i++) {
			if ([myDataString characterAtIndex:i] != ';') { // filter data with ";" char
				
				[bufferEnterStr appendFormat:@"%C",[myDataString characterAtIndex:i]];	  //appendString:myDataString];
				
			} else {
				// finish him!
				val = [bufferEnterStr copy];	
				bufferEnterStr = [[NSMutableString alloc] init];
				isOKA = YES;
			}
			
		}
		
		
		if (isOKA) { // message is ready to be sended
			
			//NSLog(val);
			
			if ([val isEqualToString:@"next"]) {
				[keynote playNextFrame];
				
			} else if ([val isEqualToString:@"previous"]) {
				[keynote playPreviousFrame];
				
			} else { // go to number section
				
				
				if (sendDifferencies) {
					if (![val isEqualToString:buffTAG]) {
						
						//	NSLog(val);
						[keynote gotoFrame:val];
						
					}
				} else { // send all from serial port
					
					[keynote gotoFrame:val];
					
				}
				
				buffTAG = [val copy]; // put in memory last info from serial
				
			}
		}
			
		[sendPort readDataInBackground];
	} else { // port closed
		[infos insertText:@"port closed\r"];
	}
	
	
	[infos setNeedsDisplay:YES];
	[infos displayIfNeeded];
}


- (void)setDifferences:(id)sender
{
	//NSLog([sender state] ? @"Checkbox is ON" : @"Checkbox is OFF");
	sendDifferencies = [sender state] ? YES : NO;	
	
}

- (IBAction)playKeynote:(id)sender {	
	[keynote play];
}




@end
